	TITLE           "ICOM IC2KL - Kenwood TS450S Interface"
	SUBTITLE        ""

;*********************************************************************************************************
; Das Interface fragt den Zustand des Kennwood TS450S regelmsig per RS232-Schnittstelle ab
; und ebersetzt die eingestellte Frequenz in einen Spannungswert zur automatischen Bandwahl
; der ICOM IC2KL endstufe.
; Kennwood - RS232 4800 Baud, 8 Bit, No Parity, 2 Stopbit mit Handshare RTC/CTS
; ICOM Spannungen zwischen 0...8 Volt
;
; 21.05.2001 Hardy Lau (DL1GLH)
;
; 28.05.2001 - LED aus bei erfolgreicher Kommunikation waehrend Sendebetrieb
;
;*********************************************************************************************************

		Processor       16F84
		Radix   DEC
		EXPAND

		include         "p16f84.inc"

_ResetVector	set	0x00
_IntVector	set	0x04

TRUE	equ	1
FALSE	equ     0

#define	LED	PORTA,3		; RA3

;*********************************************************************************************************
;

	ORG     _ResetVector
	goto    Start
;
	
	ORG     _IntVector
	goto    Interrupt
;

	include		"rs232.asm"


	cblock
		a_temp
		b_temp
		c_temp
		zehn_mhz
		ein_mhz
		zehn_mem
		ein_mem
		sendetgerade
		funktion
		split
		zaehler
	endc

;
;*********************************************************************************************************
;  Unterprogramm zur D/A-Ausgabe auf Port RA0..RA2 (3 Bit)
;
;  Wert der Ausgabe im Arbeitsregister W  
;*********************************************************************************************************
a_aus:  
	xorlw	0xff			; W invertieren und Bit 0...2 ausmaskieren
	andlw	0x07			
	movwf	a_temp
	
	bcf	STATUS,RP0		; Bank 0 fr PORTA-Zugriff
	movlw	0x07			; Falls PORTA,0 ... PORTA,2 auf Ausgabe dann "1" ausgeben.
	iorwf	PORTA,1

	bsf	STATUS,RP0		; Bank 1 fr TRISA-Zugriff
	movf	TRISA,0			; TRISA,0 ... TRISA,2 entsprechend "W" setzen
	andlw	0xF8
	iorwf	a_temp,0
	movwf	TRISA

	return

;
;*********************************************************************************************************
;  Unterprogramm zum Senden eines Zeichens
;
;  Wert der Ausgabe im Arbeitsregister W  
;*********************************************************************************************************
sende:
	movwf	TxReg
	call    PutChar
	btfsc   _txmtProgress
	goto    $-1                     ; Loop Until Transmission Over, User Can Perform Other Jobs

	movlw	150			; Verzoegerung "2tes Stopbit"
	movwf	a_temp

	decfsz	a_temp,1
	goto	$-1

	return

;
;*********************************************************************************************************
;  Unterprogramm zum Empfangen eines Zeichens (mit Timeout)
;
;  Zurueck mit Zeichen in RxReg und W = Timeout (0 = Zeichen empfangen, 1 = Timeout)
;*********************************************************************************************************
empfange:
	clrf	a_temp			; Timeoutwerte einstellen
	clrf	b_temp
	movlw	6			; knappe Sekunde
	movwf	c_temp

	call    GetChar                 ; wait for a byte reception

e_schl:
	btfsc   _rcvOver                ; _rcvOver Gets Cleared when a Byte Is Received (in RxReg)
	goto    e_timeout_check         ; USER can perform other jobs here, can poll _rcvOver bit
	retlw	0			; Zeichen empfangen

e_timeout_check:
	decfsz	a_temp,1		; a_temp herunterzaehlen
	goto	e_schl
	decfsz	b_temp,1		; b_temp herunterzaehlen
	goto	e_schl
	decfsz	c_temp,1		; c_temp herunterzaehlen, bei 0 Timeout !!!
	goto	e_schl
	retlw	1			; Timeout !!!

;
;*********************************************************************************************************
;  Unterprogramm zum einschalten der LED
;
;  
;*********************************************************************************************************
led_ein:
	bsf	STATUS,RP0		; Bank1 fr TRISA-Zugriff
	bcf	LED
	bcf	STATUS,RP0		; Bank0 fr PORTA-Zugriff
	bsf	LED
	return

;
;*********************************************************************************************************
;  Unterprogramm zum ausschalten der LED
;
;  
;*********************************************************************************************************
led_aus:
	bsf	STATUS,RP0		; Bank1 fr TRISA-Zugriff
	bcf	LED
	bcf	STATUS,RP0		; Bank0 fr PORTA-Zugriff
	bcf	LED
	return

;
;*********************************************************************************************************
;  Unterprogramm zur Vorbereitug auf das Senden (Handshake)
;
;  
;*********************************************************************************************************
moechte_senden:
   if _USE_RTSCTS
	bcf     STATUS,RP0
	bsf     _RTS                    ; Half duplex mode, transmission mode, ask host not to send data
	btfsc   _CTS                    ; Check CTS signal if host ready to accept data
	goto    $-1
   endif
	return

;
;*********************************************************************************************************
;  Unterprogramm zur Vorbereitung auf das Empfangen (Handshake)
;
;  
;*********************************************************************************************************
moechte_empfangen:
   if _USE_RTSCTS
	bcf     STATUS,RP0
	bcf     _RTS                    ; ready to accept data from host
   endif
	return

;
;*********************************************************************************************************
;  Unterprogramm zur Nachbereitung nach dem Empfangen (Handshake)
;
;  
;*********************************************************************************************************
nicht_bereit:
   if _USE_RTSCTS
	bcf     STATUS,RP0
	bsf     _RTS                    ; Half duplex mode, ask host not to send data
   endif
	return

;
;*********************************************************************************************************
;  Unterprogramm zum Senden des Befehls "IF;"
;
;  
;*********************************************************************************************************
kommando_if:
	call 	moechte_senden
	movlw	'I'
	call	sende
	movlw	'F'
	call	sende
	movlw	';'
	call	sende
	return

;
;*********************************************************************************************************
;  Unterprogramm zum Senden des Befehls "FA;" oder "FB;"
;
;  Jenach wert von "funktion" wird "FA;" oder "FB;" gesendet.
;*********************************************************************************************************
kommando_fx:
	call 	moechte_senden
	movlw	'F'
	call	sende

	movlw 	'B'
	btfsc	funktion,0		
	movlw	'A'
	call	sende

	movlw	';'
	call	sende
	return

;
;*********************************************************************************************************
;  Unterprogramm zum Senden des Befehls "MR1 XY;"
;
;  
;*********************************************************************************************************
kommando_mr:
	call 	moechte_senden
	movlw	'M'
	call	sende
	movlw	'R'
	call	sende
	movlw	'1'
	call	sende
	movlw	' '
	call	sende
	
	movf	zehn_mem,0
	call	sende
	movf	ein_mem,0
	call 	sende

	movlw	';'
	call	sende
	return

;
;*********************************************************************************************************
;  Unterprogramm zum Auswerten der Antwort "IF...;"
;  
;*********************************************************************************************************
antwort_if:
	call 	moechte_empfangen

	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'I'
	btfss	STATUS,Z
	goto	a_fehler_char		; Antwort falsch - nicht "I"
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'F'
	btfss	STATUS,Z
	goto	a_fehler_char		; Antwort falsch - nicht "F"
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'0'
	btfss	STATUS,Z
	goto	a_fehler_char		; Antwort falsch - nicht "0"	
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'0'
	btfss	STATUS,Z
	goto	a_fehler_char		; Antwort falsch - nicht "0"
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'0'
	btfss	STATUS,Z
	goto	a_fehler_char		; Antwort falsch - nicht "0"
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	zehn_mhz		; Zehnerstelle MHz ablegen
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	ein_mhz			; Einerstelle MHz ablegen
	;
	;
	movlw	19			; 19 Zeichen "weglesen"
	movwf	zaehler
	;
a_loop_1:
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	decfsz	zaehler,1
	goto	a_loop_1
	;
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	zehn_mem		; Zehnerstelle Speicherkanal ablegen
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	ein_mem			; Einerstelle Speicherkanal ablegen
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	movwf	sendetgerade		; Sendet er gerade ???
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	funktion		; Funktion ablegen
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	split			; Split-Einstellung ablegen
	;
	;
	movlw	4			; 4 Zeichen "weglesen"
	movwf	zaehler
	;
a_loop_2:
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	decfsz	zaehler,1
	goto	a_loop_2
	;
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	';'
	btfss	STATUS,Z
	goto	a_fehler_char		; Antwort falsch - nicht ";"
	;
	call	nicht_bereit
	retlw	0			; Alles OK
	

a_fehler_char:
	; lesen bis timeout !!!
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	a_fehler_time		; Weiterlesen bis Timeout
	movf	RxReg,0
	sublw	';'
	btfss	STATUS,Z
	goto	a_fehler_char		; oder Antwort ";"
	
a_fehler_time:
	call	nicht_bereit
	retlw	1

;
;*********************************************************************************************************
;  Unterprogramm zum Auswerten der Antwort "MR...;"
;  
;*********************************************************************************************************
antwort_mr:
	call 	moechte_empfangen

	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'M'
	btfss	STATUS,Z
	goto	am_fehler_char		; Antwort falsch - nicht "M"
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'R'
	btfss	STATUS,Z
	goto	am_fehler_char		; Antwort falsch - nicht "R"
	;
	;
	movlw	7			; 7 Zeichen "weglesen"
	movwf	zaehler
	;
am_loop_1:
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	decfsz	zaehler,1
	goto	am_loop_1
	;
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	zehn_mhz		; Zehnerstelle MHz ablegen
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	ein_mhz			; Einerstelle MHz ablegen
	;
	;
	movlw	12			; 12 Zeichen "weglesen"
	movwf	zaehler
	;
am_loop_2:
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	decfsz	zaehler,1
	goto	am_loop_2
	;
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	';'
	btfss	STATUS,Z
	goto	am_fehler_char		; Antwort falsch - nicht ";"
	;
	call	nicht_bereit
	retlw	0			; Alles OK

am_fehler_char:
	; lesen bis timeout !!!
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	am_fehler_time		; Weiterlesen bis Timeout
	movf	RxReg,0
	sublw	';'
	btfss	STATUS,Z
	goto	am_fehler_char		; oder Antwort ";"
	
am_fehler_time:
	call	nicht_bereit
	retlw	1

;
;*********************************************************************************************************
;  Unterprogramm zum Auswerten der Antwort "FX...;"
;  
;*********************************************************************************************************
antwort_fx:
	call 	moechte_empfangen

	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	'F'
	btfss	STATUS,Z
	goto	af_fehler_char		; Antwort falsch - nicht "F"
	;
	;
	movlw	4			; 4 Zeichen "weglesen"
	movwf	zaehler
	;
af_loop_1:
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	decfsz	zaehler,1
	goto	af_loop_1
	;
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	zehn_mhz		; Zehnerstelle MHz ablegen
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	movf	RxReg,0
	movwf	ein_mhz			; Einerstelle MHz ablegen
	;
	;
	movlw	6			; 6 Zeichen "weglesen"
	movwf	zaehler
	;
af_loop_2:
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Timeout aufgetreten - von Vorne beginnen	
	;
	decfsz	zaehler,1
	goto	af_loop_2
	;
	;
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Timeout aufgetreten - von Vorne beginnen		
	movf	RxReg,0
	sublw	';'
	btfss	STATUS,Z
	goto	af_fehler_char		; Antwort falsch - nicht ";"
	;
	call	nicht_bereit
	retlw	0			; Alles OK

af_fehler_char:
	; lesen bis timeout !!!
	call	empfange
	iorlw	0
	btfss	STATUS,Z	
	goto	af_fehler_time		; Weiterlesen bis Timeout
	movf	RxReg,0
	sublw	';'
	btfss	STATUS,Z
	goto	af_fehler_char		; oder Antwort ";"
	
af_fehler_time:
	call	nicht_bereit
	retlw	1

;
;*********************************************************************************************************
; mhz_zu_dawert
;
; Wandelt zehn_mhz und ein_mhz in einen D/A-Wert 0..7 passend fuer ICOM IC2KL 
; und gibt diesen aus.
;*********************************************************************************************************
mhz_zu_dawert:
	movlw	'0'
	subwf	zehn_mhz,0
	btfss	STATUS,Z
	goto 	mhz_g_9

;0-9 MHz
	movlw	'1'
	subwf	ein_mhz,0
	btfss	STATUS,Z
	goto	mhz_g_1
	movlw	7			; 1,8 MHz = 7 = 7,0-8,0 V
	goto	mhz_aus

mhz_g_1:
;2-9 MHz	
	movlw	'3'
	subwf	ein_mhz,0
	btfss	STATUS,Z
	goto	mhz_g_3
	movlw	6			; 3,5 MHz = 6 = 6,0-6,5 V
	goto	mhz_aus

mhz_g_3:
;4-9 MHz	
	movlw	5			; 7 MHz = 5 = 5,0-5,5 V
	goto 	mhz_aus

mhz_g_9:
	movlw	'1'
	subwf	zehn_mhz,0
	btfss	STATUS,Z
	goto 	mhz_g_19

;10-19 MHz
	movlw	'0'
	subwf	ein_mhz,0
	btfss	STATUS,Z
	goto	mhz_g_10
	movlw	0			; 10 MHz = 0 = 0-1,2 V
	goto	mhz_aus

mhz_g_10:
;11-19 MHz	
	movlw	'4'
	subwf	ein_mhz,0
	btfss	STATUS,Z
	goto	mhz_g_14
	movlw	4			; 14 MHz = 4 = 4,0-4,5 V
	goto	mhz_aus

mhz_g_14:
;15-19 MHz	
	movlw	3			; 18 MHz = 3 = 3,0-3,5 V
	goto 	mhz_aus

mhz_g_19:
;20-29 MHz
	movlw	'1'
	subwf	ein_mhz,0
	btfss	STATUS,Z
	goto	mhz_g_21
	movlw	3			; 21 MHz = 3 = 3,0-3,5 V
	goto	mhz_aus

mhz_g_21:
;22-29 MHz	
	movlw	2			; 24,28,29 MHz = 2 = 2,0-2,5 V
	
mhz_aus:
	call	a_aus
	return

;
;*********************************************************************************************************
; Hauptprogramm
;
;  
;*********************************************************************************************************

Start:
	call    InitSerialPort
;
mainloop:

	call	led_ein
	call	kommando_if		; "IF;"-Fragen
	call	antwort_if		; "IF...;"-Antwort lesen
	iorlw	0
	btfss	STATUS,Z	
	goto	mainloop		; Fehler aufgetreten - von Vorne beginnen
;
; IF-Antwort war OK
;
	btfsc	sendetgerade,0		; Falls gerade auf Sendung nichts machen!
	goto	fertig

	btfss	funktion,1		; Wenn funktion|2 dann Speicherbetrieb
	goto	vfo

mem:	call	kommando_mr		; MEM-Betrieb - Frequenz des Speicherkanales lesen
	call	antwort_mr
	iorlw	0
	btfss	STATUS,Z	
	goto	mainloop		; Fehler aufgetreten - von Vorne beginnen
	goto	ausgabe

vfo:	btfss	split,0			; Split ?
	goto	ausgabe
	call	kommando_fx		; VFO-Split-Betrieb - Frequenz des anderen VFOs lesen
	call	antwort_fx		
	iorlw	0
	btfss	STATUS,Z	
	goto	mainloop		; Fehler aufgetreten - von Vorne beginnen

ausgabe:
	call	mhz_zu_dawert		; Wert der Fequenz in Band bersetzen und als Spannung ausgeben

fertig:	
	call	led_aus			

	call 	empfange		; einfach eine verzoegerung !!!

	goto    mainloop

;*********************************************************************************************************


	END




